""" Djib Network Constants """

KMS_DEV_ENDPOINT: str = "https://devkms.djib.io"
""" Djib Key Manager Service Dev-net Providers """

KMS_MAIN_ENDPOINT: str = "https://kms.djib.io"
""" Djib Key Manager Service Main-net Providers """

PEERS_ENDPOINT: str = "https://nodes.djib.io/api/peers/"
""" Djib Peers List """

